#pragma once

#include <iostream>
#include <iomanip>

using namespace std;

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace ArrayBasedStructures
		{
			namespace OOP
			{
				/**
				 * \brief Fronta implementovan v poli s vyuitm %OOP
				 *
				 * Implementace fronty v poli jako kruhov buffer.
				 * Implementace v poli znamen, e ukazatel je v tomto ppad realizovn jako slo typu int
				 * a ukazatel je tud bn index v poli.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				class Queue
				{
				public:
					/**
					 * Konstruktor. Velikost fronty bude nastavena na vchoz velikost.
					 */
					Queue();

					/**
					 * Konstruktor
					 *
					 * @param QueueSize Velikost fronty
					 */
					Queue(const int QueueSize);

					/**
					 * Destruktor.
					 */
					~Queue();

					/**
					 * Vloen prvku na konec fronty.
					 *
					 * @param X Vkldan prvek
					 */
					void Enqueue(const int X);

					/**
					 * Vyjmut prvku z ela fronty.
					 *
					 * @return Prvek z hlavy fronty
					 */
					int Dequeue();

					/**
					 * Prvek na ele fronty.
					 * Prvek nen z ela fronty odstrann, jedn se o nedestruktivn variantu metody #Dequeue
					 *
					 * @return Prvek z hlavy fronty
					 */
					int Peek();

					/**
					 * Test je-li fronta przdn.
					 *
					 * @return Funkce vrac true pokud je fronta przdn, jinak false.
					 */
					bool IsEmpty();

					/**
					 * Test je-li fronta pln, tj. nelze vloit dal prvek.
					 *
					 * @return Funkce vrac true pokud je fronta pln, jinak false.
					 */
					bool IsFull();

					/**
					 * Smazn celho obsahu fronty.
					 */
					void Clear();
					
					/**
					 * Pomocn funkce, kter zobrazuje vnitn strukturu fronty.
					 */
					void Report();

				private:
					/**
					 * Velikost fronty.
					 */
					int QueueSize;

					/**
					 * Pole obsahujc data uloen do fronty.
					 */
					int* Items;

					/**
					 * Hlava fronty tj. ukazatel na prvn prvek fronty v poli {@link #mData}.
					 */
					int Head;

					/**
					 * Ocas fronty tj. ukazatel na prvn voln prvek za koncem fronty v poli {@link #mData}.
					 */
					int Tail;

					/**
					 * Vchoz (default) velikost fronty.
					 */
					const int DefaultQueueSize = 5;
				};
			}
		}
	}
}