#pragma once

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace ArrayBasedStructures
		{
			namespace OOPTemplate
			{
				/**
				 * \brief Zsobnk implementovan v poli jako ablona (template) %OOP
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				template <typename T>
				class Stack
				{
				public:
					/**
					 * Konstruktor. Velikost zsobnku bude nastavena na vchoz velikost.
					 */
					Stack();

					/**
					 * Konstruktor. Velikost zsobnku lze zvolit.
					 *
					 * @param StackSize Velikost zsobnku
					 */
					Stack(const int StackSize);

					/**
					 * Destruktor
					 */
					~Stack();

					/**
					 * Vloen prvku na vrchol zsobnku.
					 *
					 * @param X Vkldan prvek
					 */
					void Push(const T X);

					/**
					 * Odebrn prvku z vrcholu zsobnku.
					 *
					 * @return Prvek z vrcholu zsobnku
					 */
					T Pop();

					/**
					* Prvek na vrcholu zsobnku.
					* Prvek nen ze zsobnku odstrann, jedn se tedy o nedestruktivn variantu funkce #Pop.
					*
					* @return Prvek z vrcholu zsobnku
					*/
					T Top() const;

					/**
					 * Test je-li zsobnk przdn.
					 *
					 * @return Funkce vrac true pokud je zsobnk przdn, jinak false.
					 */
					bool IsEmpty() const;

					/**
					 * Test, je-li zsobnk zaplnn.
					 *
					 * @return Funkce vrac true pokud je zsobnk zaplnn, jinak false.
					 */
					bool IsFull() const;

					/**
					 * Smazn celho obsahu zsobnku.
					 */
					void Clear();

				private:
					/**
					 * Pole obsahujc data uloen do zsobnku.
					 */
					T* Items;

					/**
					 * Velikost zsobnku.
					 */
					int StackSize;

					/**
					 * Ukazatel zsobnku.
					 */
					int StackPointer;

					/**
					 * Vchoz (default) velikost zsobnku.
					 */
					static const int DefaultStackSize = 100;
				};


				template <typename T> Stack<T>::Stack()
				{
					this->Items = new T[DefaultStackSize];
					this->StackPointer = 0;
				}

				template <typename T> Stack<T>::Stack(const int StackSize)
				{
					this->StackSize = StackSize <= 0 ? DefaultStackSize : StackSize;
					this->Items = new char[this->StackSize];
					this->StackPointer = 0;
				}

				template <typename T> Stack<T>::~Stack()
				{
					delete[] this->Items;
				}

				template <typename T> void Stack<T>::Push(const T X)
				{
					this->Items[this->StackPointer] = X;
					this->StackPointer += 1;
				}

				template <typename T> T Stack<T>::Pop()
				{
					this->StackPointer -= 1;
					return this->Items[this->StackPointer];
				}

				template <typename T> T Stack<T>::Top() const
				{
					return this->Items[this->StackPointer - 1];
				}

				template <typename T> bool Stack<T>::IsEmpty() const
				{
					return this->StackPointer == 0;
				}

				template <typename T> bool Stack<T>::IsFull() const
				{
					return this->StackPointer >= this->StackSize;
				}

				template <typename T> void Stack<T>::Clear()
				{
					this->StackPointer = 0;
				}
			}
		}
	}
}
