#pragma once

#include <iostream>
#include <iomanip>

using namespace std;

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace ArrayBasedStructures
		{
			namespace Procedural
			{
				/**
				 * Velikost kruhovho bufferu ve kterm je fronta implementovna.
				 */
				const int CircularBufferSize = 5;

				/**
				 * Maximln velikost fronty.
				 */
				const int MaxQueueSize = CircularBufferSize - 1;

				/**
				 * \brief Fronta implementovan v poli jako kruhov buffer
				 *
				 * Implementace fronty v poli.
				 * Implementace v poli znamen, e ukazatel je v tomto ppad realizovn jako slo typu int
				 * a ukazatel je tud bn index v poli.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				struct Queue
				{
					/**
					 * Pole, kruhov buffer, obsahujc data uloen do fronty.
					 */
					int Items[CircularBufferSize];
					/**
					 * Hlava fronty. Hlava fronty ukazuje na prvn prvek fronty.
					 * Prvn prvek fronty nazvme tak elo fronty.
					 */
					int Head;
					/**
					 * Ocas fronty. Tento ukazatel bn ukazuje na posledn prvek ve front.
					 * Pi implementaci pomoc kruhovho bufferu v poli, je vhodnj, aby tento
					 * ukazatel ukazoval a za posledn prvek ve front. Ocas fronty bude tak ukazovat
					 * na prvn voln prvek v poli {@link #Data}.
					 */
					int Tail;
				};

				/**
				 * Inicializace fronty.
				 *
				 * @param Q Inicializovan fronta
				 */
				void Init(Queue& Q);

				/**
				 * Vloen prvku na konec fronty.
				 *
				 * @param Q Fronta
				 * @param X Vkldan prvek
				 */
				void Enqueue(Queue& Q, const int X);

				/**
				 * Odebrn prvku z ela fronty.
				 *
				 * @param Q Fronta
				 * @return Prvek z ela fronty
				 */
				int Dequeue(Queue& Q);

				/**
				 * Prvek na ele fronty.
				 * Prvek nen z ela fronty odstrann, jedn se o nedestruktivn variantu metody #Dequeue
				 *
				 * @param Q Fronta
				 * @return Prvek z ela fronty
				 */
				int Peek(Queue& Q);

				/**
				 * Test, je-li fronta przdn.
				 *
				 * @param Q Fronta
				 * @return Funkce vrac true pokud je fronta przdn, jinak false.
				 */
				bool IsEmpty(const Queue& Q);

				/**
				 * Test, je-li fronta pln, tj. nelze vloit dal prvek.
				 *
				 * @param Q Fronta
				 * @return Funkce vrac true pokud je fronta pln, jinak false.
				 */
				bool IsFull(const Queue& Q);

				/**
				 * Smazn celho obsahu fronty.
				 *
				 * @param Q Fronta
				 */
				void Clear(Queue& Q);

				/**
				 * Pomocn funkce, kter zobrazuje vnitn strukturu fronty.
				 *
				 * @param Q Fronta
				 */
				void Report(const Queue& Q);
			}
		}
	}
}