#pragma once

#include <cstdlib>
#include <iostream>
#include <iomanip>

using namespace std;

namespace ADSLibrary
{
	namespace DataStructures
	{
		namespace LinkedStructures
		{
			namespace Procedural
			{
				/**
				 * \brief Struktura reprezentujc poloku v seznamu {@link #ADSLibrary::DataStructures::LinkedStructures::Procedural::List}
				 */
				struct ListItem
				{
					/**
					 * Hodnota obsaen v poloce.
					 */
					int Value;
					/**
					 * Ukazatel na pedchoz poloku. Pokud takov poloka neexistuje ukazatel m hodnotu NULL.
					 */
					ListItem* Prev;
					/**
					 * Ukazatel na nsledujc  poloku. Pokud takov poloka neexistuje ukazatel m hodnotu NULL.
					 */
					ListItem* Next;
				};

				/**
				 * \brief Seznam implementovan pomoc spojovch struktur procedurlnm zpsobem
				 *
				 * Implementace obousmrnho seznamu pomoc spojovch struktur.
				 * V seznamu je implementovn jednoduch itertor.
				 *
				 * @author	Ji Dvorsk <jiri.dvorsky@vsb.cz>
				 * @date	2010 - 2015
				 */
				struct List
				{
					/**
					 * Hlava seznamu tj. ukazatel na prvn poloku seznamu.
					 */
					ListItem* Head;
					/**
					 * Ukazatel na aktuln poloku pi iteraci seznamem.
					 */
					ListItem* Current;
				};

				/**
				 * Inicializace seznamu.
				 *
				 * @param L Inicializovan seznam
				 */
				void Init(List& L);

				/**
				 * Smazn celho seznamu.
				 *
				 * @param L Seznam
				 */
				void Clear(List& L);

				/**
				 * Vloen prvku do seznamu. Prvek je vloen ped souasnou hlavu seznamu tj. stane se novou hlavou seznamu.
				 *
				 * @param L Seznam
				 * @param X Vkldan prvek
				 */
				void Insert(List& L, const int X);

				/**
				 * Odebrn prvku ze seznamu. Prvek X je nejprve v seznamu nalezen a pot je odpovdajc poloka ze seznamu vymazna.
				 *
				 * @param L Seznam
				 * @param X Odebran prvek
				 */
				void Delete(List& L, const int X);

				/**
				 * Hledn prvku v seznamu.
				 *
				 * @param L Seznam
				 * @param X Hledan prvek
				 * @return Funkce vrac true v ppad, e je prvek X nalezen, jinak vrac false.
				 */
				bool Search(const List& L, const int X);

				/**
				 * Test je-li seznam przdn.
				 *
				 * @param L Seznam
				 * @return Funkce vrac true pokud je seznam przdn, jinak false.
				 */
				bool IsEmpty(const List& L);

				/**
				 * Funkce vrac poet prvk v seznamu.
				 *
				 * @param L Seznam
				 * @return Poet prvk v seznamu.
				 */
				int Count(const List& L);

				/**
				 * Reset itertoru. Ukazatel na aktuln poloku seznamu bude ukazovat na hlavu seznamu.
				 *
				 * @param L Seznam
				 */
				void Reset(List& L);

				/**
				 * Test ukonen innosti itertoru.
				 *
				 * @param L Seznam
				 * @return Funkce vrac true pokud je iterace seznamem ukonena, jinak false.
				 */
				bool IsEnd(const List& L);

				/**
				 * Posun ukazatele aktuln poloky na dal poloku v seznamu.
				 *
				 * @param L Seznam
				 */
				void MoveNext(List& L);

				/**
				 * Vrcen prvku z aktuln poloky seznamu pi iteraci.
				 *
				 * @param L Seznam
				 * @return Funkce vrac prvek z aktuln poloky seznamu pi iteraci.
				 */
				int Current(const List& L);

				/**
				 * Pomocn funkce, kter zobrazuje vnitn strukturu seznamu.
				 *
				 * @param L Seznam
				 */
				void Report(const List& L);
			}
		}
	}
}