#pragma once 
#include "ADSLibrary\DataStructures\LinkedStructures\OOP\Queue.h"
#include "ADSLibrary\DataStructures\LinkedStructures\OOP\List.h"

class GraphInLists
{
private:
	//pole seznamu sousednuch vrcholu, n-ty seznam v poli reprezentuje seznam sousedu vrcholu s cislem n
	ADSLibrary::DataStructures::LinkedStructures::OOP::List* graph;
	int vertexCount;
	bool* visited;//pole pro ulozeni informace zda byl vrchol pri pruchodech gram navstiven ci nikoliv
	int* predecessors;//pole pro ulozeni predku vrcholu

public:
	GraphInLists(int NoOfNodes)
	{
		vertexCount = NoOfNodes;
		graph = new ADSLibrary::DataStructures::LinkedStructures::OOP::List[vertexCount + 1]();
		visited = new bool[vertexCount + 1];
		predecessors = new int[vertexCount + 1];
		resetInfo();
	}

	~GraphInLists()
	{
		delete[] graph;
		delete[] visited;
		delete[] predecessors;
	}

	//je graf strom? souvisl graf bez cyklu
	bool isTree()
	{
		if (DFS()>1)//strom mus bt souvisl, tedy mus mt jen jednu komponentu
			return false;
		else
			return(!hasCycle(1));//test na cyklus (kter vrchol vybereme pro zacatek algoritmu je jedno)
	}

	//ur, zda komponenta grafu, do kter spad vrchol s islem pedanm jako argument, obsahuje cyklus
	//upraven hledn do iky
	//v grafu je cyklus - pri pruchodu narazime na jiz navstiveny vrchol
	//pro detekci cyklu je potreba ukladat predchudce vrcholu - viz nize
	bool hasCycle(int vertex)
	{
		resetInfo();

		ADSLibrary::DataStructures::LinkedStructures::OOP::Queue foundVertexes;
		foundVertexes.Enqueue(vertex);
		visited[vertex] = true;

		int found;

		while (!foundVertexes.IsEmpty())
		{
			found = foundVertexes.Dequeue();

			for (graph[found].Reset(); !graph[found].IsEnd(); graph[found].MoveNext())
			{
				if (!visited[graph[found].Current()])//pokud vrchol nebyl navstiven, provedeme operace jako v BFS
				{
					visited[graph[found].Current()] = true;
					predecessors[graph[found].Current()] = found;//ulozime predka nove nalezeneho vrcholu
					foundVertexes.Enqueue(graph[found].Current());
				}
				else//pokud jsme narazili na navstiveny vrchol a neni to navstiveny predchudce ze ktereho jsme prisli, nalezli jsme cyklus
					if (graph[found].Current() != predecessors[found])
						return true;
			}
		}
		return false;
	}

	void AddEdge(int a, int b)
	{
		graph[a].Insert(b);
		graph[b].Insert(a);
	}

	void report()
	{
		cout << "Graf v seznamech na " << vertexCount << " vrcholech." << endl;
		cout << "hrany: " << endl;
		for (int i = 1; i < (vertexCount + 1); i++)
		{
			//reset seznamu, iterator nastaven na prvni prvek
			for (graph[i].Reset(); !graph[i].IsEnd(); graph[i].MoveNext())//cyklus for, prcohzime seznam s kontrolou isend
				if (i<graph[i].Current())//hrany vypiseme pouze jednou, vypisujeme dvojice kde prvni cislo vrcholu mensi nez druhe
					cout << "hrana z: " << i << " do: " << graph[i].Current() << endl;
		}
	}

	//prochazeni jedne kompoenenty grafu do hloubky, pocinaje vrcholem vertex
	// DFS Visit viz prednasky
	//nelze pouzit sikovne pro jednu komponentu jako BFS - nelze zde provest resetinfo (provedlo by se pri kazdem rekurzivnim zavolani na jeden vrchol)
	void DFSV(int vertex)
	{
		cout << "nalezen vrchol: " << vertex << endl;
		visited[vertex] = true;
		for (graph[vertex].Reset(); !graph[vertex].IsEnd(); graph[vertex].MoveNext())
		{
			if (!visited[graph[vertex].Current()])
				DFSV(graph[vertex].Current());
		}
	}

	//prohledavani do hloubky pro vsechny komponenty grafu, vraci jejich pocet
	//stejnym zpusobem muzeme rozsirit pro vsechny kompoenenty algoritmus hledani do sirky(misto DFSV budeme volat BFS)
	int DFS()
	{
		resetInfo();
		int componentCount = 0;

		for (int i = 1; i < (vertexCount + 1); i++)
			if (!visited[i])
			{
				componentCount++;
				cout << componentCount << ". komponenta: " << endl;
				DFSV(i);
			}

		return componentCount;
	}

	//reset hodnot v polich na vychozi hodnoty
	void resetInfo()
	{
		for (int i = 1; i < (vertexCount + 1); i++)
		{
			visited[i] = false;
			predecessors[i] = 0;
		}
	}

	//prochazeni jedne komponenty grafu do sirky, pocinaje vrcholem vertex
	void BFS(int vertex)
	{
		resetInfo();
		ADSLibrary::DataStructures::LinkedStructures::OOP::Queue foundVertexes;

		cout << "nalezen vrchol: " << vertex << endl;
		visited[vertex] = true;
		foundVertexes.Enqueue(vertex);
		
		int found;

		while (!foundVertexes.IsEmpty())
		{
			//vybereme vrchol z fronty pro zpracovani 
			found = foundVertexes.Dequeue();

			//cyklus kter projde vsechny sousedy vrcholu s cislem ulozenym v promenne found
			//nenavstivene sousedy vlozi do fronty k naslednemu zpracovani
			for (graph[found].Reset(); !graph[found].IsEnd(); graph[found].MoveNext())
			{
				if (!visited[graph[found].Current()])//aktualni soused jeste nebyl navstiven
				{
					cout << "nalezen vrchol: " << graph[found].Current() << endl;
					visited[graph[found].Current()] = true;
					foundVertexes.Enqueue(graph[found].Current());
				}
			}
		}
	}
};